﻿using System;
using System.Data.SqlClient;

namespace Simorgh.SqlServerDependency
{
    public class QueryNotification
        : IDisposable
    {
        //________________________________________________________________________

        private readonly string m_ConnectionString;
        private readonly string m_SqlSelect;
        private volatile bool m_IsDisposed = false;

        public event OnChangeEventHandler OnChange;
        //________________________________________________________________________

        public QueryNotification(string ConnectionString, string SqlSelect)
        {
            this.m_ConnectionString = ConnectionString;
            this.m_SqlSelect = SqlSelect;

            SqlDependency.Stop(this.m_ConnectionString);
            SqlDependency.Start(this.m_ConnectionString);
            this.Reset();
        }
        //________________________________________________________________________

        private void Reset()
        {
            using (var con = new System.Data.SqlClient.SqlConnection(this.m_ConnectionString))
            using (var com = new System.Data.SqlClient.SqlCommand(this.m_SqlSelect, con))
            {
                if (this.m_IsDisposed) return;
                var dependency = new SqlDependency(com);
                dependency.OnChange += this.OnChangeHandler;

                con.Open();
                com.ExecuteNonQuery();
            }
        }
        //________________________________________________________________________

        private void OnChangeHandler(object sender, SqlNotificationEventArgs e)
        {
            var handler = this.OnChange;
            if (handler != null && !this.m_IsDisposed) handler(this, e);

            var dependency = sender as SqlDependency;
            if (dependency != null) dependency.OnChange -= this.OnChangeHandler;

            if (e.Info == SqlNotificationInfo.Invalid) return;
            this.Reset();
        }
        //________________________________________________________________________

        public virtual void Dispose()
        {
            if (this.m_IsDisposed) return;
            this.m_IsDisposed = true;
            SqlDependency.Stop(this.m_ConnectionString);
        }
        //________________________________________________________________________
    }
}
